/**
 * SPDX-FileCopyrightText: (c) 2023 Liferay, Inc. https://liferay.com
 * SPDX-License-Identifier: LGPL-2.1-or-later OR LicenseRef-Liferay-DXP-EULA-2.0.0-2023-06
 */

import {API} from '@liferay/object-js-components-web';
import React, {useCallback, useState} from 'react';
import ReactFlow, {
	Background,
	Connection,
	ConnectionLineType,
	ConnectionMode,
	Controls,
	Edge,
	MiniMap,
	Node,
	isNode,
	useStore,
} from 'react-flow-renderer';

import {ModalAddObjectRelationship} from '../../ObjectRelationship/ModalAddObjectRelationship';
import {getUpdatedModelBuilderStructurePayload} from '../../ViewObjectDefinitions/objectDefinitionUtil';
import DefaultObjectRelationshipEdge from '../Edges/DefaultObjectRelationshipEdge';
import SelfObjectRelationshipEdge from '../Edges/SelfObjectRelationshipEdge';
import {useObjectFolderContext} from '../ModelBuilderContext/objectFolderContext';
import {TYPES} from '../ModelBuilderContext/typesEnum';
import {EmptyNode} from '../ObjectDefinitionNode/EmptyNode';
import {ObjectDefinitionNode} from '../ObjectDefinitionNode/ObjectDefinitionNode';

import './Diagram.scss';

const NODE_TYPES = {
	emptyNode: EmptyNode,
	objectDefinitionNode: ObjectDefinitionNode,
};

const EDGE_TYPES = {
	defaultObjectRelationshipEdge: DefaultObjectRelationshipEdge,
	selfObjectRelationshipEdge: SelfObjectRelationshipEdge,
};

function DiagramBuilder({
	setShowModal,
}: {
	setShowModal: (value: React.SetStateAction<ModelBuilderModals>) => void;
}) {
	const [
		{
			baseResourceURL,
			elements,
			isLoadingObjectFolder,
			selectedObjectFolder,
			showChangesSaved,
		},
		dispatch,
	] = useObjectFolderContext();

	const [
		showAddObjectRelationshipModal,
		setShowAddObjectRelationshipModal,
	] = useState(false);
	const [
		newObjectRelationshipSourceNodeProps,
		setNewObjectRelationshipSourceNodeProps,
	] = useState<{
		parameterRequired: boolean;
		sourceNode: {
			erc: string;
		};
		targetNode: {
			erc: string;
		};
	}>();

	const emptyNode = [
		{
			data: {
				setShowModal,
			},
			id: 'empty',
			position: {
				x: 400,
				y: 400,
			},
			type: 'emptyNode',
		},
	];

	const store = useStore();

	const onConnect = useCallback(
		(connection: Connection | Edge) => {
			const sourceNode = elements.find(
				(node) => isNode(node) && node.id === connection.source
			) as Node<ObjectDefinitionNodeData>;

			const targetNode = elements.find(
				(node) => isNode(node) && node.id === connection.target
			) as Node<ObjectDefinitionNodeData>;

			if (
				(sourceNode.data?.modifiable === false &&
					targetNode.data?.modifiable === false) ||
				(sourceNode.data?.system && targetNode.data?.system) ||
				sourceNode.data?.storageType === 'salesforce' ||
				targetNode.data?.storageType === 'salesforce' ||
				targetNode.data?.name === 'Address' ||
				sourceNode.data?.linkedObjectDefinition
			) {
				return;
			}

			setShowAddObjectRelationshipModal(true);
			setNewObjectRelationshipSourceNodeProps({
				parameterRequired: sourceNode?.data?.parameterRequired!,
				sourceNode: {
					erc: sourceNode?.data?.externalReferenceCode!,
				},
				targetNode: {
					erc: targetNode?.data?.externalReferenceCode!,
				},
			});
		},
		[elements]
	);

	const onNodeDragStop = async (node: Node<ObjectDefinitionNodeData>) => {
		const objectFolder = await API.getObjectFolderByExternalReferenceCode(
			selectedObjectFolder.externalReferenceCode
		);

		const updatedObjectFolderItems = objectFolder.objectFolderItems.map(
			(objectFolderItem) => {
				if (
					objectFolderItem.objectDefinitionExternalReferenceCode ===
					node.data?.externalReferenceCode
				) {
					return {
						...objectFolderItem,
						positionX: node.position.x,
						positionY: node.position.y,
					};
				}

				return objectFolderItem;
			}
		);

		const updatedObjectFolder = {
			externalReferenceCode: selectedObjectFolder.externalReferenceCode,
			id: selectedObjectFolder.id,
			label: selectedObjectFolder.label,
			name: selectedObjectFolder.name,
			objectFolderItems: updatedObjectFolderItems,
		};

		await API.putObjectFolderByExternalReferenceCode(updatedObjectFolder);

		const {edges, nodes} = store.getState();

		dispatch({
			payload: {
				newObjectDefinitionNodePosition: {
					x: node.position.x,
					y: node.position.y,
				},
				objectDefinitionNodes: nodes,
				objectRelationshipEdges: edges,
				updatedObjectDefinitionNodeId: node.data?.id as number,
			},
			type: TYPES.SET_SELECTED_OBJECT_DEFINITION_NODE_POSITION,
		});

		if (!showChangesSaved) {
			dispatch({
				payload: {updatedShowChangesSaved: true},
				type: TYPES.SET_SHOW_CHANGES_SAVED,
			});
		}
	};

	const updateModelBuilderStructure = async (
		newObjectRelationshipId: number
	) => {
		const payload = await getUpdatedModelBuilderStructurePayload(
			selectedObjectFolder.name
		);

		dispatch({
			payload: {
				...payload,
				rightSidebarType: 'objectRelationshipDetails',
				selectedObjectRelationshipEdgeId: newObjectRelationshipId,
			},
			type: TYPES.UPDATE_MODEL_BUILDER_STRUCTURE,
		});
	};

	return (
		<div className="lfr-objects__model-builder-diagram-area">
			{showAddObjectRelationshipModal && (
				<ModalAddObjectRelationship
					baseResourceURL={baseResourceURL}
					handleOnClose={() =>
						setShowAddObjectRelationshipModal(false)
					}
					hasDefinedObjectDefinitionTarget
					objectDefinitionExternalReferenceCode1={
						newObjectRelationshipSourceNodeProps?.sourceNode.erc!
					}
					objectDefinitionExternalReferenceCode2={
						newObjectRelationshipSourceNodeProps?.targetNode.erc!
					}
					objectRelationshipParameterRequired={
						newObjectRelationshipSourceNodeProps?.parameterRequired!
					}
					onAfterSubmit={(newObjectRelationshipId: number) =>
						updateModelBuilderStructure(newObjectRelationshipId)
					}
					reload={false}
				/>
			)}

			<ReactFlow
				connectionLineStyle={{stroke: '#0B5FFF'}}
				connectionLineType={ConnectionLineType.SmoothStep}
				connectionMode={ConnectionMode.Loose}
				edgeTypes={EDGE_TYPES}
				elements={
					!isLoadingObjectFolder
						? elements.length
							? elements
							: emptyNode
						: []
				}
				minZoom={0.1}
				nodeTypes={NODE_TYPES}
				onConnect={onConnect}
				onNodeDragStop={(_, node) => onNodeDragStop(node)}
			>
				<Background color="#C0C1C3" gap={18} size={1} />

				{!isLoadingObjectFolder ? (
					<>
						<Controls showInteractive={false} />
						<MiniMap />
					</>
				) : (
					<div className="lfr-objects__model-builder-diagram-area-loading">
						<span
							aria-hidden="true"
							className="loading-animation-lg loading-animation-primary loading-animation-squares"
						/>
					</div>
				)}
			</ReactFlow>
		</div>
	);
}

export default DiagramBuilder;
